@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2023, Even Rouault <even dot rouault at spatialys dot com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "s100.h"
@@ -433,8 +417,9 @@ std::string S100ReadMetadata(GDALDataset *poDS, const std::string &osFilename,
             const char *pszVal = poAttr->ReadAsString();
             if (pszVal && pszVal[0])
             {
-                osMetadataFile = CPLFormFilename(CPLGetPath(osFilename.c_str()),
-                                                 pszVal, nullptr);
+                osMetadataFile = CPLFormFilenameSafe(
+                    CPLGetPathSafe(osFilename.c_str()).c_str(), pszVal,
+                    nullptr);
                 VSIStatBufL sStat;
                 if (VSIStatL(osMetadataFile.c_str(), &sStat) != 0)
                 {
@@ -443,9 +428,9 @@ std::string S100ReadMetadata(GDALDataset *poDS, const std::string &osFilename,
                     // but the actual filename does not start with "MD_"...
                     if (STARTS_WITH(pszVal, "MD_"))
                     {
-                        osMetadataFile =
-                            CPLFormFilename(CPLGetPath(osFilename.c_str()),
-                                            pszVal + strlen("MD_"), nullptr);
+                        osMetadataFile = CPLFormFilenameSafe(
+                            CPLGetPathSafe(osFilename.c_str()).c_str(),
+                            pszVal + strlen("MD_"), nullptr);
                         if (VSIStatL(osMetadataFile.c_str(), &sStat) != 0)
                         {
                             osMetadataFile.clear();
