@@ -143,8 +143,8 @@ MFFTiledBand::~MFFTiledBand()
 CPLErr MFFTiledBand::IReadBlock(int nBlockXOff, int nBlockYOff, void *pImage)
 
 {
-    const int nTilesPerRow = (nRasterXSize + nBlockXSize - 1) / nBlockXSize;
-    const int nWordSize = GDALGetDataTypeSize(eDataType) / 8;
+    const int nTilesPerRow = DIV_ROUND_UP(nRasterXSize, nBlockXSize);
+    const int nWordSize = GDALGetDataTypeSizeBytes(eDataType);
     const int nBlockSize = nWordSize * nBlockXSize * nBlockYSize;
 
     const vsi_l_offset nOffset =
@@ -774,16 +774,13 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
     /* -------------------------------------------------------------------- */
     /*      Read the directory to find matching band files.                 */
     /* -------------------------------------------------------------------- */
-    char *const pszTargetPath =
-        CPLStrdup(CPLGetPathSafe(poOpenInfo->pszFilename).c_str());
-    char *const pszTargetBase =
-        CPLStrdup(CPLGetBasenameSafe(poOpenInfo->pszFilename).c_str());
-    char **papszDirFiles =
-        VSIReadDir(CPLGetPathSafe(poOpenInfo->pszFilename).c_str());
-    if (papszDirFiles == nullptr)
+    const std::string osTargetPath = CPLGetPathSafe(poOpenInfo->pszFilename);
+    const std::string osTargetBase =
+        CPLGetBasenameSafe(poOpenInfo->pszFilename);
+    const CPLStringList aosDirFiles(
+        VSIReadDir(CPLGetPathSafe(poOpenInfo->pszFilename).c_str()));
+    if (aosDirFiles.empty())
     {
-        CPLFree(pszTargetPath);
-        CPLFree(pszTargetBase);
         return nullptr;
     }
 
@@ -793,14 +790,13 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
         /* Find the next raw band file. */
 
         int i = 0;  // Used after for.
-        for (; papszDirFiles[i] != nullptr; i++)
+        for (; i < aosDirFiles.size(); i++)
         {
-            if (!EQUAL(CPLGetBasenameSafe(papszDirFiles[i]).c_str(),
-                       pszTargetBase))
+            if (!EQUAL(CPLGetBasenameSafe(aosDirFiles[i]).c_str(),
+                       osTargetBase.c_str()))
                 continue;
 
-            const std::string osExtension =
-                CPLGetExtensionSafe(papszDirFiles[i]);
+            const std::string osExtension = CPLGetExtensionSafe(aosDirFiles[i]);
             if (osExtension.size() >= 2 &&
                 isdigit(static_cast<unsigned char>(osExtension[1])) &&
                 atoi(osExtension.c_str() + 1) == nRawBand &&
@@ -808,12 +804,12 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
                 break;
         }
 
-        if (papszDirFiles[i] == nullptr)
+        if (i == aosDirFiles.size())
             break;
 
         /* open the file for required level of access */
         const std::string osRawFilename =
-            CPLFormFilenameSafe(pszTargetPath, papszDirFiles[i], nullptr);
+            CPLFormFilenameSafe(osTargetPath.c_str(), aosDirFiles[i], nullptr);
 
         VSILFILE *fpRaw = nullptr;
         if (poOpenInfo->eAccess == GA_Update)
@@ -832,7 +828,7 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
             CSLAddString(poDS->m_papszFileList, osRawFilename.c_str());
 
         GDALDataType eDataType = GDT_Unknown;
-        const std::string osExt = CPLGetExtensionSafe(papszDirFiles[i]);
+        const std::string osExt = CPLGetExtensionSafe(aosDirFiles[i]);
         if (pszRefinedType != nullptr)
         {
             if (EQUAL(pszRefinedType, "C*4"))
@@ -918,6 +914,13 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
 
         if (bTiled)
         {
+            if (nTileXSize > INT_MAX / nTileYSize / nPixelOffset)
+            {
+                CPLError(CE_Failure, CPLE_AppDefined, "Too large tile");
+                CPL_IGNORE_RET_VAL(VSIFCloseL(fpRaw));
+                return nullptr;
+            }
+
             poBand = std::make_unique<MFFTiledBand>(poDS.get(), nBand, fpRaw,
                                                     nTileXSize, nTileYSize,
                                                     eDataType, eByteOrder);
@@ -943,10 +946,6 @@ GDALDataset *MFFDataset::Open(GDALOpenInfo *poOpenInfo)
         poDS->SetBand(nBand, std::move(poBand));
     }
 
-    CPLFree(pszTargetPath);
-    CPLFree(pszTargetBase);
-    CSLDestroy(papszDirFiles);
-
     /* -------------------------------------------------------------------- */
     /*      Check if we have bands.                                         */
     /* -------------------------------------------------------------------- */
