@@ -7,23 +7,7 @@
  ******************************************************************************
  * Copyright (c) 2014, Even Rouault <even dot rouault at spatialys dot com>
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_port.h"
@@ -38,16 +22,6 @@
 #include "ogr_p.h"
 #include "ogr_spatialref.h"
 
-/************************************************************************/
-/*                         OGRCompoundCurve()                           */
-/************************************************************************/
-
-/**
- * \brief Create an empty compound curve.
- */
-
-OGRCompoundCurve::OGRCompoundCurve() = default;
-
 /************************************************************************/
 /*             OGRCompoundCurve( const OGRCompoundCurve& )              */
 /************************************************************************/
@@ -63,12 +37,6 @@ OGRCompoundCurve::OGRCompoundCurve() = default;
 
 OGRCompoundCurve::OGRCompoundCurve(const OGRCompoundCurve &) = default;
 
-/************************************************************************/
-/*                         ~OGRCompoundCurve()                          */
-/************************************************************************/
-
-OGRCompoundCurve::~OGRCompoundCurve() = default;
-
 /************************************************************************/
 /*                 operator=( const OGRCompoundCurve&)                  */
 /************************************************************************/
@@ -100,7 +68,16 @@ OGRCompoundCurve &OGRCompoundCurve::operator=(const OGRCompoundCurve &other)
 OGRCompoundCurve *OGRCompoundCurve::clone() const
 
 {
-    return new (std::nothrow) OGRCompoundCurve(*this);
+    auto ret = new (std::nothrow) OGRCompoundCurve(*this);
+    if (ret)
+    {
+        if (ret->WkbSize() != WkbSize())
+        {
+            delete ret;
+            ret = nullptr;
+        }
+    }
+    return ret;
 }
 
 /************************************************************************/
@@ -380,19 +357,19 @@ OGRBoolean OGRCompoundCurve::Equals(const OGRGeometry *poOther) const
 /*                       setCoordinateDimension()                       */
 /************************************************************************/
 
-void OGRCompoundCurve::setCoordinateDimension(int nNewDimension)
+bool OGRCompoundCurve::setCoordinateDimension(int nNewDimension)
 {
-    oCC.setCoordinateDimension(this, nNewDimension);
+    return oCC.setCoordinateDimension(this, nNewDimension);
 }
 
-void OGRCompoundCurve::set3D(OGRBoolean bIs3D)
+bool OGRCompoundCurve::set3D(OGRBoolean bIs3D)
 {
-    oCC.set3D(this, bIs3D);
+    return oCC.set3D(this, bIs3D);
 }
 
-void OGRCompoundCurve::setMeasured(OGRBoolean bIsMeasured)
+bool OGRCompoundCurve::setMeasured(OGRBoolean bIsMeasured)
 {
-    oCC.setMeasured(this, bIsMeasured);
+    return oCC.setMeasured(this, bIsMeasured);
 }
 
 /************************************************************************/
@@ -667,9 +644,9 @@ void OGRCompoundCurve::flattenTo2D()
 /*                              segmentize()                            */
 /************************************************************************/
 
-void OGRCompoundCurve::segmentize(double dfMaxLength)
+bool OGRCompoundCurve::segmentize(double dfMaxLength)
 {
-    oCC.segmentize(dfMaxLength);
+    return oCC.segmentize(dfMaxLength);
 }
 
 /************************************************************************/
@@ -946,11 +923,25 @@ double OGRCompoundCurve::get_GeodesicArea(
     if (!poSRSOverride)
         poSRSOverride = getSpatialReference();
 
-    OGRLineString *poLS = CurveToLine();
-    const double dfArea = poLS->get_GeodesicArea(poSRSOverride);
-    delete poLS;
+    auto poLS = std::unique_ptr<OGRLineString>(CurveToLine());
+    return poLS->get_GeodesicArea(poSRSOverride);
+}
 
-    return dfArea;
+/************************************************************************/
+/*                        get_GeodesicLength()                          */
+/************************************************************************/
+
+double OGRCompoundCurve::get_GeodesicLength(
+    const OGRSpatialReference *poSRSOverride) const
+{
+    if (IsEmpty())
+        return 0;
+
+    if (!poSRSOverride)
+        poSRSOverride = getSpatialReference();
+
+    auto poLS = std::unique_ptr<OGRLineString>(CurveToLine());
+    return poLS->get_GeodesicLength(poSRSOverride);
 }
 
 /************************************************************************/
@@ -970,3 +961,39 @@ double OGRCompoundCurve::get_AreaOfCurveSegments() const
     }
     return dfArea;
 }
+
+/************************************************************************/
+/*                           hasEmptyParts()                            */
+/************************************************************************/
+
+bool OGRCompoundCurve::hasEmptyParts() const
+{
+    return oCC.hasEmptyParts();
+}
+
+/************************************************************************/
+/*                          removeEmptyParts()                          */
+/************************************************************************/
+
+void OGRCompoundCurve::removeEmptyParts()
+{
+    oCC.removeEmptyParts();
+}
+
+/************************************************************************/
+/*                           reversePoints()                            */
+/************************************************************************/
+
+/**
+ * \brief Reverse point order.
+ *
+ * This method updates the points in this curve in place
+ * reversing the point ordering (first for last, etc) and component ordering.
+ *
+ * @since 3.10
+ */
+void OGRCompoundCurve::reversePoints()
+
+{
+    oCC.reversePoints();
+}
