@@ -18,6 +18,7 @@
 #include "cpl_time.h"
 #include "ogr_p.h"
 #include "ograrrowarrayhelper.h"
+#include "ogrlayerarrow.h"
 #include "ogr_recordbatch.h"
 
 #include "ogr_flatgeobuf.h"
@@ -1493,6 +1494,8 @@ begin:
     }
 
     const GIntBig nFeatureIdxStart = m_featuresPos;
+    const bool bDateTimeAsString = m_aosArrowArrayStreamOptions.FetchBool(
+        GAS_OPT_DATETIME_AS_STRING, false);
 
     const uint32_t nMemLimit = OGRArrowArrayHelper::GetMemLimit();
     while (iFeat < sHelper.m_nMaxBatchSize)
@@ -1863,6 +1866,58 @@ begin:
                         offset += sizeof(double);
                         break;
 
+                    case ColumnType::DateTime:
+                    {
+                        if (!bDateTimeAsString)
+                        {
+                            if (offset + sizeof(uint32_t) > size)
+                            {
+                                CPLErrorInvalidSize("datetime length ");
+                                goto error;
+                            }
+                            uint32_t len;
+                            memcpy(&len, data + offset, sizeof(int32_t));
+                            CPL_LSBPTR32(&len);
+                            offset += sizeof(uint32_t);
+                            if (len > size - offset || len > 32)
+                            {
+                                CPLErrorInvalidSize("datetime value");
+                                goto error;
+                            }
+                            if (!isIgnored)
+                            {
+                                OGRField ogrField;
+                                if (ParseDateTime(
+                                        reinterpret_cast<const char *>(data +
+                                                                       offset),
+                                        len, &ogrField))
+                                {
+                                    sHelper.SetDateTime(
+                                        psArray, iFeat, brokenDown,
+                                        sHelper.m_anTZFlags[i], ogrField);
+                                }
+                                else
+                                {
+                                    char str[32 + 1];
+                                    memcpy(str, data + offset, len);
+                                    str[len] = '\0';
+                                    if (OGRParseDate(str, &ogrField, 0))
+                                    {
+                                        sHelper.SetDateTime(
+                                            psArray, iFeat, brokenDown,
+                                            sHelper.m_anTZFlags[i], ogrField);
+                                    }
+                                }
+                            }
+                            offset += len;
+                            break;
+                        }
+                        else
+                        {
+                            [[fallthrough]];
+                        }
+                    }
+
                     case ColumnType::String:
                     case ColumnType::Json:
                     case ColumnType::Binary:
@@ -1908,50 +1963,6 @@ begin:
                         offset += len;
                         break;
                     }
-
-                    case ColumnType::DateTime:
-                    {
-                        if (offset + sizeof(uint32_t) > size)
-                        {
-                            CPLErrorInvalidSize("datetime length ");
-                            goto error;
-                        }
-                        uint32_t len;
-                        memcpy(&len, data + offset, sizeof(int32_t));
-                        CPL_LSBPTR32(&len);
-                        offset += sizeof(uint32_t);
-                        if (len > size - offset || len > 32)
-                        {
-                            CPLErrorInvalidSize("datetime value");
-                            goto error;
-                        }
-                        if (!isIgnored)
-                        {
-                            OGRField ogrField;
-                            if (ParseDateTime(reinterpret_cast<const char *>(
-                                                  data + offset),
-                                              len, &ogrField))
-                            {
-                                sHelper.SetDateTime(psArray, iFeat, brokenDown,
-                                                    sHelper.m_anTZFlags[i],
-                                                    ogrField);
-                            }
-                            else
-                            {
-                                char str[32 + 1];
-                                memcpy(str, data + offset, len);
-                                str[len] = '\0';
-                                if (OGRParseDate(str, &ogrField, 0))
-                                {
-                                    sHelper.SetDateTime(
-                                        psArray, iFeat, brokenDown,
-                                        sHelper.m_anTZFlags[i], ogrField);
-                                }
-                            }
-                        }
-                        offset += len;
-                        break;
-                    }
                 }
             }
         }
@@ -2386,14 +2397,15 @@ OGRErr OGRFlatGeobufLayer::ICreateFeature(OGRFeature *poNewFeature)
     }
 }
 
-OGRErr OGRFlatGeobufLayer::GetExtent(OGREnvelope *psExtent, int bForce)
+OGRErr OGRFlatGeobufLayer::IGetExtent(int iGeomField, OGREnvelope *psExtent,
+                                      bool bForce)
 {
     if (m_sExtent.IsInit())
     {
         *psExtent = m_sExtent;
         return OGRERR_NONE;
     }
-    return OGRLayer::GetExtent(psExtent, bForce);
+    return OGRLayer::IGetExtent(iGeomField, psExtent, bForce);
 }
 
 int OGRFlatGeobufLayer::TestCapability(const char *pszCap)
@@ -2444,14 +2456,14 @@ void OGRFlatGeobufLayer::ResetReading()
 std::string OGRFlatGeobufLayer::GetTempFilePath(const CPLString &fileName,
                                                 CSLConstList papszOptions)
 {
-    const CPLString osDirname(CPLGetPath(fileName.c_str()));
-    const CPLString osBasename(CPLGetBasename(fileName.c_str()));
+    const CPLString osDirname(CPLGetPathSafe(fileName.c_str()));
+    const CPLString osBasename(CPLGetBasenameSafe(fileName.c_str()));
     const char *pszTempDir = CSLFetchNameValue(papszOptions, "TEMPORARY_DIR");
     std::string osTempFile =
-        pszTempDir ? CPLFormFilename(pszTempDir, osBasename, nullptr)
+        pszTempDir ? CPLFormFilenameSafe(pszTempDir, osBasename, nullptr)
         : (STARTS_WITH(fileName, "/vsi") && !STARTS_WITH(fileName, "/vsimem/"))
-            ? CPLGenerateTempFilename(osBasename)
-            : CPLFormFilename(osDirname, osBasename, nullptr);
+            ? CPLGenerateTempFilenameSafe(osBasename)
+            : CPLFormFilenameSafe(osDirname, osBasename, nullptr);
     osTempFile += "_temp.fgb";
     return osTempFile;
 }
