@@ -9,23 +9,7 @@
  * Copyright (c) 2017, Even Rouault
  * Copyright (c) 2017, Planet Labs
  *
- * Permission is hereby granted, free of charge, to any person obtaining a
- * copy of this software and associated documentation files (the "Software"),
- * to deal in the Software without restriction, including without limitation
- * the rights to use, copy, modify, merge, publish, distribute, sublicense,
- * and/or sell copies of the Software, and to permit persons to whom the
- * Software is furnished to do so, subject to the following conditions:
- *
- * The above copyright notice and this permission notice shall be included
- * in all copies or substantial portions of the Software.
- *
- * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
- * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
- * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
- * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
- * DEALINGS IN THE SOFTWARE.
+ * SPDX-License-Identifier: MIT
  ****************************************************************************/
 
 #include "cpl_http.h"
@@ -36,6 +20,7 @@
 
 #include "cpl_conv.h"
 #include "cpl_error.h"
+#include "cpl_json.h"
 #include "cpl_string.h"
 
 /* ==================================================================== */
@@ -58,43 +43,6 @@
 
 #define GOOGLE_AUTH_URL "https://accounts.google.com/o/oauth2"
 
-/************************************************************************/
-/*                          ParseSimpleJson()                           */
-/*                                                                      */
-/*      Return a string list of name/value pairs extracted from a       */
-/*      JSON doc.  The Google OAuth2 web service returns simple JSON    */
-/*      responses.  The parsing as done currently is very fragile       */
-/*      and depends on JSON documents being in a very very simple       */
-/*      form.                                                           */
-/************************************************************************/
-
-static CPLStringList ParseSimpleJson(const char *pszJson)
-
-{
-    /* -------------------------------------------------------------------- */
-    /*      We are expecting simple documents like the following with no    */
-    /*      hierarchy or complex structure.                                 */
-    /* -------------------------------------------------------------------- */
-    /*
-        {
-            "access_token":"1/fFBGRNJru1FQd44AzqT3Zg",
-            "expires_in":3920,
-            "token_type":"Bearer"
-        }
-    */
-
-    CPLStringList oWords(
-        CSLTokenizeString2(pszJson, " \n\t,:{}", CSLT_HONOURSTRINGS));
-    CPLStringList oNameValue;
-
-    for (int i = 0; i < oWords.size(); i += 2)
-    {
-        oNameValue.SetNameValue(oWords[i], oWords[i + 1]);
-    }
-
-    return oNameValue;
-}
-
 /************************************************************************/
 /*                      GOA2GetAuthorizationURL()                       */
 /************************************************************************/
@@ -248,7 +196,7 @@ char CPL_DLL *GOA2GetRefreshToken(const char *pszAuthToken,
     }
     */
     CPLStringList oResponse =
-        ParseSimpleJson(reinterpret_cast<char *>(psResult->pabyData));
+        CPLParseKeyValueJson(reinterpret_cast<char *>(psResult->pabyData));
     CPLHTTPDestroyResult(psResult);
 
     CPLString osAccessToken = oResponse.FetchNameValueDef("access_token", "");
@@ -307,7 +255,7 @@ static char **GOA2ProcessResponse(CPLHTTPResult *psResult)
     }
     */
     CPLStringList oResponse =
-        ParseSimpleJson(reinterpret_cast<char *>(psResult->pabyData));
+        CPLParseKeyValueJson(reinterpret_cast<char *>(psResult->pabyData));
     CPLHTTPDestroyResult(psResult);
 
     CPLString osAccessToken = oResponse.FetchNameValueDef("access_token", "");
