@@ -582,7 +582,7 @@ VSITarFilesystemHandler::CreateReader(const char *pszTarFileName)
 
 VSIVirtualHandle *VSITarFilesystemHandler::Open(const char *pszFilename,
                                                 const char *pszAccess,
-                                                bool /* bSetError */,
+                                                bool bSetError,
                                                 CSLConstList /* papszOptions */)
 {
 
@@ -594,7 +594,8 @@ VSIVirtualHandle *VSITarFilesystemHandler::Open(const char *pszFilename,
     }
 
     CPLString osTarInFileName;
-    char *tarFilename = SplitFilename(pszFilename, osTarInFileName, TRUE);
+    char *tarFilename =
+        SplitFilename(pszFilename, osTarInFileName, true, bSetError);
     if (tarFilename == nullptr)
         return nullptr;
 
