@@ -591,11 +591,12 @@ char **VSIWebHDFSFSHandler::GetFileList(const char *pszDirname,
         osDelegationParam = "&delegation=" + osDelegationParam;
     std::string osURL =
         osBaseURL + "?op=LISTSTATUS" + osUsernameParam + osDelegationParam;
+    const CPLStringList aosHTTPOptions(CPLHTTPGetOptionsFromEnv(pszDirname));
 
     CURL *hCurlHandle = curl_easy_init();
 
     struct curl_slist *headers =
-        VSICurlSetOptions(hCurlHandle, osURL.c_str(), nullptr);
+        VSICurlSetOptions(hCurlHandle, osURL.c_str(), aosHTTPOptions.List());
 
     WriteFuncStruct sWriteFuncData;
     VSICURLInitWriteFuncStruct(&sWriteFuncData, nullptr, nullptr, nullptr);
@@ -694,13 +695,14 @@ int VSIWebHDFSFSHandler::Unlink(const char *pszFilename)
         osDelegationParam = "&delegation=" + osDelegationParam;
     std::string osURL =
         osBaseURL + "?op=DELETE" + osUsernameParam + osDelegationParam;
+    const CPLStringList aosHTTPOptions(CPLHTTPGetOptionsFromEnv(pszFilename));
 
     CURL *hCurlHandle = curl_easy_init();
 
     unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_CUSTOMREQUEST, "DELETE");
 
     struct curl_slist *headers =
-        VSICurlSetOptions(hCurlHandle, osURL.c_str(), nullptr);
+        VSICurlSetOptions(hCurlHandle, osURL.c_str(), aosHTTPOptions.List());
 
     WriteFuncStruct sWriteFuncData;
     VSICURLInitWriteFuncStruct(&sWriteFuncData, nullptr, nullptr, nullptr);
@@ -817,13 +819,14 @@ int VSIWebHDFSFSHandler::Mkdir(const char *pszDirname, long nMode)
         osURL += "&permission=";
         osURL += CPLSPrintf("%o", static_cast<int>(nMode));
     }
+    const CPLStringList aosHTTPOptions(CPLHTTPGetOptionsFromEnv(pszDirname));
 
     CURL *hCurlHandle = curl_easy_init();
 
     unchecked_curl_easy_setopt(hCurlHandle, CURLOPT_CUSTOMREQUEST, "PUT");
 
     struct curl_slist *headers =
-        VSICurlSetOptions(hCurlHandle, osURL.c_str(), nullptr);
+        VSICurlSetOptions(hCurlHandle, osURL.c_str(), aosHTTPOptions.List());
 
     WriteFuncStruct sWriteFuncData;
     VSICURLInitWriteFuncStruct(&sWriteFuncData, nullptr, nullptr, nullptr);
